import 'dart:io';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hive/hive.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';
import 'package:path_provider/path_provider.dart';

import 'package:minehunter/config/default_global_settings.dart';
import 'package:minehunter/config/theme.dart';
import 'package:minehunter/cubit/game_cubit.dart';
import 'package:minehunter/cubit/nav_cubit.dart';
import 'package:minehunter/cubit/settings_game_cubit.dart';
import 'package:minehunter/cubit/settings_global_cubit.dart';
import 'package:minehunter/cubit/theme_cubit.dart';
import 'package:minehunter/ui/skeleton.dart';

void main() async {
  // Initialize packages
  WidgetsFlutterBinding.ensureInitialized();
  await EasyLocalization.ensureInitialized();
  final Directory tmpDir = await getTemporaryDirectory();
  Hive.init(tmpDir.toString());
  HydratedBloc.storage = await HydratedStorage.build(
    storageDirectory: tmpDir,
  );

  SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp])
      .then((value) => runApp(EasyLocalization(
            path: 'assets/translations',
            supportedLocales: const <Locale>[
              Locale('en'),
              Locale('fr'),
            ],
            fallbackLocale: const Locale('en'),
            useFallbackTranslations: true,
            child: const MyApp(),
          )));
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    final List<String> assets = getImagesAssets();
    for (String asset in assets) {
      precacheImage(AssetImage(asset), context);
    }

    return MultiBlocProvider(
      providers: [
        BlocProvider<NavCubit>(create: (context) => NavCubit()),
        BlocProvider<ThemeCubit>(create: (context) => ThemeCubit()),
        BlocProvider<GameCubit>(create: (context) => GameCubit()),
        BlocProvider<GlobalSettingsCubit>(create: (context) => GlobalSettingsCubit()),
        BlocProvider<GameSettingsCubit>(create: (context) => GameSettingsCubit()),
      ],
      child: BlocBuilder<ThemeCubit, ThemeModeState>(
        builder: (BuildContext context, ThemeModeState state) {
          return MaterialApp(
            title: 'Minehunter',
            home: const SkeletonScreen(),

            // Theme stuff
            theme: lightTheme,
            darkTheme: darkTheme,
            themeMode: state.themeMode,

            // Localization stuff
            localizationsDelegates: context.localizationDelegates,
            supportedLocales: context.supportedLocales,
            locale: context.locale,
            debugShowCheckedModeBanner: false,
          );
        },
      ),
    );
  }

  List<String> getImagesAssets() {
    final List<String> assets = [];

    final List<String> gameImages = [
      'button_back',
      'button_delete_saved_game',
      'button_resume_game',
      'button_start',
      'game_fail',
      'game_win',
      'placeholder',
    ];

    for (String image in gameImages) {
      assets.add('assets/ui/$image.png');
    }

    final List<String> skinImages = [
      'button_mark_mine_off',
      'button_mark_mine_on',
      'empty',
      'indicator_report_off',
      'indicator_report_on',
      'indicator_walk_off',
      'indicator_walk_on',
      'tile_flag_ko',
      'tile_flag_ok',
      'tile_flag',
      'tile_mine_not_found',
      'tile_mine',
      'tile_unknown',
    ];
    for (int value = 0; value <= 8; value++) {
      skinImages.add('tile_$value');
    }

    for (String skin in DefaultGlobalSettings.allowedSkinValues) {
      for (String image in skinImages) {
        assets.add('assets/skins/${skin}_$image.png');
      }
    }

    return assets;
  }
}
